#!/usr/bin/env python
"""This houses the Stuff class.
    
Use stuff_scraper in order to gather a list of
stuffs. For testing, the reverse Geolocator is at 
nominatim.openstreetmap.org.
    
Example usage::

    >>> from stuff_scraper import StuffScraper
    >>> stuffs = StuffScraper('montreal', 5).stuffs
    >>> print(stuffs[0].thing)  # Title
    Meubles / furniture
    >>> stuffs[0].find_coordinates() # pass precise=True in constructor
    >>> print(stuffs[0].coordinates) # to automatically fetch coordinates
    ['45.617854', '-73.633931']
"""
import requests, re
from geopy.geocoders import Nominatim
from bs4 import BeautifulSoup


class Stuff(object):
    """A freestuff Craigslist object.
    
    Fill this object with the information from
    a craigslist page. (There is no price attribute,
    because it is designed for invaluable things).
    The precise coordinates are not initially set,
    because they require significantly more requests.
    See class method get_coordinates().
    
    Attributes:
        - thing -- title of object passed explicitly
        - url -- constructed from url, implicit
        - image -- passed explicitly
        - user_location -- passed explicitly, requires clean up
        - coordinates -- array of longitude and latitude
    Keyword arguements:
        - thing -- 
        - url -- 
        - location -- 
        - image -- 
        - user_location -- must conform to valid craiglist url
    """    
    def __init__(self, thing, url, location, image, user_location):
        self.thing = thing
        self.url = 'http://' + user_location + '.craigslist.ca' + url
        self.location = location
        self.image = image
        self.user_location = user_location


    def __str__(self):
        """Print stuff summary."""
        return "what: %s \n where: %s \n link: %s \n image: %s" \
                % (self.thing, self.location, self.url, self.image)


    def find_coordinates(self):
        """Get and set longitude and Latitude
        
        Scrape individual posting page, if no
        coordinates are found, cascade precision
        (try location, try user_location, or set
        to zero).  Returns an array, first 
        latitude and then longitude.
        """
        self.coordinates = []
        geolocator = Nominatim()
        follow_this = self.url
        follow_page = requests.get(follow_this)
        follow_soup = BeautifulSoup(follow_page.text, "html.parser")
        location = follow_soup.find("div", class_="viewposting")
        if location is not None: # Get from Page
            lat = location['data-latitude']
            lon = location['data-longitude']
        else:
            try: # Get from posted location
                lat = geolocator.geocode(self.location).latitude
                lon = geolocator.geocode(self.location).longitude
            except:
                try: # Get from user locatoin
                    lat = geolocator.geocode(self.user_location).latitude
                    lon = geolocator.geocode(self.user_location).longitude
                except:
                    lat = 0 #38.9047 # This is DC
                    lon = 0 #-77.0164
        self.coordinates.append(lat)
        self.coordinates.append(lon)



#!/usr/bin/env python
"""Chart where free things are.

The StuffCharter class is a wrapper around the folium 
openstreetmap python object, which in turn generates a 
leaflet map.

Example usage:

    >>> from stuff_scraper import StuffScraper
    >>> from stuff_charter import StuffCharter
    >>> stuffs = StuffScraper('montreal', 5, precise=True).stuffs
    >>> treasure_map = StuffCharter(stuffs)
    call save_map(path) generate html map
    >>> treasure_map.save_test_map() # saves map in current dir
    BEWARNED, this map is likely inaccurate:
    Craigslist denizens care not for computer-precision
"""
import os, re

from geopy.geocoders import Nominatim
from bs4 import BeautifulSoup
import requests, folium, webbrowser
from folium.element import IFrame


class StuffCharter:
    """Post folium map of freestuffs.

    After constructing Mappify map object, call
    create_map and pass in map_path in order to create 
    the HTML map.
    
    Attributes:
        - treasure_map -- an OSM folium map object
        - stuffs -- list of free stuff
        - user_location -- the user's location
        - start_coordinates -- the origin coordinates for city
        - zoom -- default map zoom
        
    Keyword arguments:
        - stuffs -- a list of stuff objects
        - address -- for an optional map marker of the user address.
        - do_create_map -- set to False to override modify attributes
                           before create_map.
        - is_testing -- use to test module from commandline
        - is_flask -- automatically create map for treasure-map
        - zoom -- the map default zoom level 
    """
    
    def __init__(self, stuffs, address=None, zoom=13, 
                 do_create_map=True, 
                 is_testing=False, is_flask=False):
        self.user_location = stuffs[0].user_location
        self.start_coordinates = self.find_city_center(self.user_location)
        self.zoom = zoom
        self.stuffs = stuffs
        self.radius = 500
        self.address = address 
        if do_create_map:
            self.create_map(is_testing, is_flask)
        
    def create_map(self, is_testing=False, is_flask=False):
        """Create a folium Map object, treasure_map.
        
        treasure_map can be used to save an html leaflet map.
        This method is called automatically on __init__ unless
        do_create_map is set to False.
        
        Keyword arguments:
            - is_testing -- creates a map in webmap directory
            - is_flask -- creates a flask map
        """
        map_osm = folium.Map([self.start_coordinates[0], self.start_coordinates[1]], zoom_start=self.zoom) 
        for stuff in self.stuffs:
            place = stuff.location  
            thing = stuff.thing  # thing title
            url = stuff.url
            image = stuff.image
            color = self.sort_stuff(stuff.thing)   # Map marker's color
            name = """
                    <link rel='stylesheet' type='text/css' 
                    href='https://maxcdn.bootstrapcdn.com/bootstrap/3.3.6/css/bootstrap.min.css'>
                    <img src='%s' height='auto' width='160px' />
                    <h3>%s</h3>
                    <h4>%s</h4>
                    <a href='%s' target='_blank'>View Posting in New Tab</a>
                   """ % (image, thing, place, url)
            # TODO: Contigency Plan for 0, 0?
            lat = stuff.coordinates[0] # Latitude
            lon = stuff.coordinates[1] # Longitude
            popup = folium.Popup(IFrame(name, width=200, height=300),
                                 max_width=3000)
            folium.CircleMarker([lat, lon], radius=self.radius, popup=popup,
                                fill_color=color, fill_opacity=0.2).add_to(map_osm)
            self.radius -= 10 # Diminishing order
        self.treasure_map = map_osm
        self.add_address(self.address)
        if is_testing:
            self.save_test_map()
        elif is_flask:
            self.save_flask_map()
        else:
            print("call save_map(path) to generate html map")
    
    def save_test_map(self):
        """Create html map in current directory.
        
        Should have python -m http.server running in directory
        """
        path = os.getcwd()
        self.treasure_map.save(os.path.join(path, 'treasure_map.html')) 
        print("BEWARNED, this map is likely inaccurate:\nCraigslist denizens care not for computer-precision")
        # webbrowser.open_new_tab("localhost:8000/webmap/findit.html") # Open the map in a tab
        
        
    def save_flask_map(self):
        """Create html map in flask server."""
        folium_figure = self.treasure_map.get_root()
        folium_figure.header._children['bootstrap'] = folium.element.CssLink('static', 'css', 'style.css') #'/static/css/style.css'
        folium_figure.header._children['Woops'] = folium.element.CssLink('static', 'css', 'map.css') # Why woops?
        self.treasure_map.save(os.path.join('treasuremap', 'templates', 'raw_map.html')) 
        
        
    def save_map(self, map_path, css_path=None): # make **argv
        """Create html map in map_path.
        
        Keyword arguments:
            - map_path -- the path to create_map in
            - css_path -- the path to override css 
                          (defaults to bootstrap via folium)
        """
        path = os.getcwd()
        if not os.path.exists(os.path.join(path, map_path)):
            os.makedirs(os.path.join(path, map_path))
        if css_path is not None:
            folium_figure = self.treasure_map.get_root() # Leaflet Style conflicts with custom Bootstrap
            folium_figure.header._children['Woops'] = folium.element.CssLink(css_path)
        self.treasure_map.save(map_path)

        
    def find_city_center(self, location):
        """Return city center longitude latitude."""
        geolocator = Nominatim()
        if re.match("montreal", location, re.I):
            coord = [45.5088, -73.5878] 
        elif re.match("newyork", location, re.I):
            coord = [40.7127, -74.0058] 
        elif re.match("toronto", location, re.I):
            coord = [43.7, -79.4000] 
        elif re.match("washingtondc", location, re.I):
            coord = [38.9047, -77.0164]
        elif re.match("vancouver", location, re.I):
            coord = [49.2827, -123.1207]
        elif re.match("sanfrancisco", location, re.I):
            coord = [37.773972, -122.431297]
        else:
            try:
                findit = geolocator.geocode(location) # Last resort
                lat = findit.latitude                 
                lon = findit.longitude                
                coord = [lat, lon]
            except:
                coord = [0,0] # This is a bit silly, nulle island
        return coord

    def add_address(self, _address):
        """Add address to folium map"""
        self.address = _address
        if _address != None:
            geolocator = Nominatim()
            try:
                add_lat = geolocator.geocode(self.address).latitude
                add_lon = geolocator.geocode(self.address).longitude
            except:
                add_lat = 0
                add_lon = 0
            pop_up = self.address + str(add_lat) + str(add_lon)
            folium.Marker(location=[add_lat, add_lon],popup=self.address,
                          icon=folium.Icon(color='red',icon='home')
                          ).add_to(self.treasure_map)        


    def sort_stuff(self, stuff): 
        """Return a color according to regex search.
        
        1. Furniture pattern, red 
        2. Electronics pattern, blue
        3. Miscellaneous pattern, black 
        4. no match, white
        
        sort_stuff will return  with the first pattern found in 
        that order.
        
        TODO:
            - Set and patterns as modifiable attributes.
        """
        PATTERN_1 = "(wood|shelf|shelves|table|chair|scrap|desk|oak|pine|armoire|dresser)"
        PATTERN_2 = "(tv|screen|écran|speakers|wire|electronic|saw|headphones|arduino|print|television)" #search NOT match
        PATTERN_3 = "(book|games|cool|guide|box)"

        COLOR_1 = "#FF0000" #red 
        COLOR_2 = "#3186cc" #blue
        COLOR_3 = "#000000" #black
        
        COLOR_DEFAULT = "white"
        if re.search(PATTERN_1, stuff, re.I):
            color = COLOR_1 #red #  TODO: set as Variable
        elif re.search(PATTERN_2, stuff, re.I): #the end all
            color = COLOR_2 #blue at once
        elif re.search(PATTERN_3, stuff, re.I):
            color = COLOR_3 #black
        else:
            color = COLOR_DEFAULT #white
        # color = "#ffff00"
        return color



import os
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


requirements = open('requirements.txt').read().splitlines()


setup(
    name='freestuffs',
    description='Find free stuff near you!',
    version='0.1.5',
    author='Fenimore Love',
    license='MIT',
    url='https://github.com/fenimore/freestuffs',
    packages=['freestuffs',],
    include_package_data=True,
    install_requires=requirements,
    long_description=read('README.rst'),
)



#!/usr/bin/env python
"""This module is a Craigslist scraper.

Example usage::

    >>> from stuff_scraper import StuffScraper
    >>> stuffs = StuffScraper('montreal', 5).stuffs
    >>> print(stuffs[0].thing) # Print title
    Meubles / furniture
"""
import requests, re
from bs4 import BeautifulSoup
from unidecode import unidecode

from freestuffs.stuff import Stuff


class StuffScraper:
    """The freestuffs Craigslist scraper.

    Compile parrellel lists of stuff attributes
    in order to store a freestuffs list, with an
    option for including stuff coordinates.

    Attributes:
        - stuffs -- a list of stuff objects
        - soup -- bs4 soup of Craiglist page
        - place -- the city to search, in Craigslist friendly format
        - locs, things, images, urls -- stuff attributes lists
        - quantity -- how many stuffs gathered

    Keyword arguments:
        - _quantity -- how many stuffs to gather
        - precise -- A boolean to explicitly use geolocator
                     and crawl individual posting URL
        - use_cl -- user input for place
    """
    def __init__(self, place, _quantity, precise=False, use_cl=False):
        """Scrape Craigslist for a list of stuff.

        Keyword arguments:
            - place -- the city to search, in Craigslist friendly format
            - _quantity -- how many stuffs to gather
            - precise -- A boolean to explicitly use geolocator
                         and crawl individual posting URL
            - use_cl -- ask for place from user input
        """
        if use_cl:
            place = self.setup_place()
        _url = 'http://' + place +'.craigslist.com/search/zip'
        try:
            free_stuff_page = requests.get(_url)
            _soup = BeautifulSoup(free_stuff_page.text, "html.parser")
        except:
            _soup = "something when wrong" # Something informative
        self.soup = _soup
        self.quantity = int(_quantity)
        self.locs = self.get_locations(place, self.soup) # locations, needs user place for fine tuning
        self.urls = self.get_urls(self.soup)
        self.things = self.get_things(self.soup) # titles
        self.images = self.get_images(self.soup)  # I can't believe this works..
        self.stuffs = [Stuff(self.things[x], self.urls[x], self.locs[x],
                           self.images[x], place)
                           for x in range(0, self.quantity)]
        if precise:
            for stuff in self.stuffs:
                stuff.find_coordinates()
        # TODO: Print how long this takes...

    def __repr__(self):
        result = str(self.quantity)
        result += " " + str(len(self.locs))
        result += " " + str(len(self.things))
        result += " " + str(len(self.images))
        result += " " + str(len(self.urls))
        return result

    def setup_place(self):
        """Take cl input of user location."""
        user_place = input("What major city are you near? (or, 'help') ")
        if user_place == "help":
            print("craigslist serves many major cities, \
            and the peripheral neighborhoods, try something\
             like 'montreal' or 'newyork'\n It's gotta be \
             one word (no spaces) or funny characters, visit\
              the craigslist.org site for your cities 'name'.\
              \nAlso, the mappify module currently only works with montreal")
            user_place = input("What major city are you near? ")
        return user_place


    def get_things(self, _soup):
        """Scrape titles.

            Keyword arguments:
            - soup - bs4 object of a Craiglist freestuffs page
        """
        free_things = []
        for node in _soup.find_all("a", class_="hdrlnk"):
            _thing = node.get_text()
            free_things.append(_thing)
        return free_things

    def get_locations(self, user_place, _soup):
        """Scape locations.

           Returns a list of locations, more or less
           precise. Concatnate user_place to string
           in order to aid geolocator in case of
           duplicate location names in world. Yikes.

           Keyword arguments:
               - user_place -- the city, in Craigslist format
               - soup -- bs4 object of a Craiglist freestuffs page
        """
        free_locations = []
        user_location = self.refine_city_name(user_place)
        for loc_node in _soup.find_all("span", class_="result-hood"):
            if loc_node is None: # Some places have no where
                _loc = user_location + ", Somewhere" # +", Somewhere"
            else:
                _loc = loc_node.text.strip('() ')
                _loc = unidecode(_loc) # unicode!
                _loc = _loc + ", " + user_location
            free_locations.append(_loc)
        return free_locations


    def get_urls(self, _soup):
        """Scrape stuff urls.

        Keyword arguments:
            - soup - bs4 object of a Craiglist freestuffs page
        """
        free_urls = []
        for row in _soup.find_all("a", class_="hdrlnk"):
            _url = row['href']
            free_urls.append(_url)
        return free_urls


    def get_images(self, _soup):
        """Scrape images.

        Uses wikpedia No-image image if no image is found.

        Keyword arguments:
            - soup - bs4 object of a Craiglist freestuffs page
        """
        free_images = []
        for row in _soup.find_all("a", class_="result-image"):
            try:
                _img = str(row['data-ids']) # Take that Craig!
                _img = _img[2:19] # eats up the first image ID
                _img = _img.replace(',', '')
                _img = "https://images.craigslist.org/" + _img + "_300x300.jpg" # this took me forever
            except:
                _img = "http://upload.wikimedia.org/wikipedia/commons/a/ac/No_image_available.svg" # No-image image
            free_images.append(_img)
        return free_images

    def refine_city_name(self, user_place):
        """Refine location for two-word cities."""
        if user_place == 'newyork': # does this have to capitalized?
            loc = 'New York' # For tweeting
        elif user_place == 'washingtondc':
            loc = 'Washington D.C.'
        elif user_place == 'sanfrancisco':
            loc = 'San Francisco, USA'
        else:
            loc = user_place
        return loc



"""Craiglist friendly city names.

Run this script inorder to print out a dict of
Craiglist city names, for use in creating a valid url.

Attributes:
    - CITIES -- A dict of cities, with the human-friendly
                name as the key
                and url friendly name as the value.
"""
from bs4 import BeautifulSoup
import requests

CITIES = {'sarasota-bradenton': 'sarasota', 'dallas / fort worth': 'dallas', 'manila': 'manila', 'corpus christi': 'corpuschristi', 'panama': 'panama', 'houma': 'houma', 'jerusalem': 'jerusalem', 'winston-salem': 'winstonsalem', 'lakeland': 'lakeland', 'eastern shore': 'easternshore', 'lansing': 'lansing', 'outer banks': 'outerbanks', 'albuquerque': 'albuquerque', 'albany ': 'albanyga', 'luxembourg': 'luxembourg', 'galveston': 'galveston', 'west bank': 'ramallah', 'cagayan de oro': 'cdo', 'lubbock': 'lubbock', 'ciudad juarez': 'juarez', 'southwest michigan': 'swmi', 'mattoon-charleston': 'mattoon', 'saginaw-midland-baycity': 'saginaw', 'huntington-ashland': 'huntington', 'meadville': 'meadville', 'ventura county': 'ventura', 'hattiesburg': 'hattiesburg', 'plattsburgh-adirondacks': 'plattsburgh', 'trois-rivieres': 'troisrivieres', 'lehigh valley': 'allentown', 'medford-ashland': 'medford', 'south bend / michiana': 'southbend', 'scottsbluff / panhandle': 'scottsbluff', 'huntsville / decatur': 'huntsville', 'dublin': 'dublin', 'farmington': 'farmington', 'northwest OK': 'enid', 'susanville': 'susanville', 'prague': 'prague', 'western slope': 'westslope', 'philadelphia': 'philadelphia', 'fortaleza': 'fortaleza', 'lake charles': 'lakecharles', 'strasbourg': 'strasbourg', 'venezuela': 'caracas', 'zurich': 'zurich', 'valdosta': 'valdosta', 'salem': 'salem', 'sendai': 'sendai', 'gold country': 'goldcountry', 'fresno / madera': 'fresno', 'south jersey': 'southjersey', 'bemidji': 'bemidji', 'provo / orem': 'provo', 'salina': 'salina', 'cambridge, UK': 'cambridge', 'aberdeen': 'aberdeen', 'tyler / east TX': 'easttexas', 'fort collins / north CO': 'fortcollins', 'singapore': 'singapore', 'tri-cities': 'tricities', 'oklahoma city': 'oklahomacity', 'st joseph': 'stjoseph', 'east oregon': 'eastoregon', 'belgium': 'brussels', 'niagara region': 'niagara', 'daytona beach': 'daytona', 'muncie / anderson': 'muncie', 'beaumont / port arthur': 'beaumont', 'caribbean islands': 'caribbean', 'northern panhandle': 'wheeling', 'toledo': 'toledo', 'asheville': 'asheville', 'treasure coast': 'treasure', 'springfield ': 'springfieldil', 'central louisiana': 'cenla', 'yuba-sutter': 'yubasutter', 'lancaster': 'lancaster', 'sao paulo': 'saopaulo', 'southwest VA': 'swva', 'twin falls': 'twinfalls', 'gulfport / biloxi': 'gulfport', 'new hampshire': 'nh', 'stillwater': 'stillwater', 'U.S. virgin islands': 'virgin', 'fredericksburg': 'fredericksburg', 'red deer': 'reddeer', 'albany': 'albany', 'washington': 'washingtondc', 'rockford': 'rockford', 'panama city': 'panamacity', 'zanesville / cambridge': 'zanesville', 'visalia-tulare': 'visalia', 'austin': 'austin', 'fargo / moorhead': 'fargo', 'duluth / superior': 'duluth', 'acapulco': 'acapulco', 'bellingham': 'bellingham', 'north central FL': 'lakecity', 'ghana': 'accra', 'porto': 'porto', 'virgin islands': 'virgin', 'lexington': 'lexington', 'baltimore': 'baltimore', 'bhubaneswar': 'bhubaneswar', 'kokomo': 'kokomo', 'reno / tahoe': 'reno', 'sardinia': 'sardinia', 'chongqing': 'chongqing', 'lewiston / clarkston': 'lewiston', 'mason city': 'masoncity', 'guam-micronesia': 'micronesia', 'chatham-kent': 'chatham', 'brasilia': 'brasilia', 'sapporo': 'sapporo', 'new york city': 'newyork', 'curitiba': 'curitiba', 'richmond ': 'richmondin', 'augusta': 'augusta', "xi'an": 'xian', 'napoli / campania': 'naples', 'chicago': 'chicago', 'phoenix': 'phoenix', 'chengdu': 'chengdu', 'helena': 'helena', 'bloomington': 'bloomington', 'hilton head': 'hiltonhead', 'south coast': 'southcoast', 'kootenays': 'kootenays', 'essex': 'essex', 'baleares': 'baleares', 'liverpool': 'liverpool', 'loire valley': 'loire', 'eau claire': 'eauclaire', 'humboldt county': 'humboldt', 'dothan': 'dothan', 'fukuoka': 'fukuoka', 'kamloops': 'kamloops', 'west virginia (old)': 'wv', 'sicilia': 'sicily', 'muskegon': 'muskegon', 'lafayette': 'lafayette', 'east idaho': 'eastidaho', 'tasmania': 'hobart', 'sydney': 'sydney', 'richmond': 'richmond', 'tunisia': 'tunis', 'kaiserslautern': 'kaiserslautern', 'cariboo': 'cariboo', 'nagoya': 'nagoya', 'poconos': 'poconos', 'lausanne': 'lausanne', 'rapid city / west SD': 'rapidcity', 'oaxaca': 'oaxaca', 'sunshine coast': 'sunshine', 'northwest GA': 'nwga', 'toulouse': 'toulouse', 'raleigh / durham / CH': 'raleigh', 'brainerd': 'brainerd', 'darwin': 'darwin', 'show low': 'showlow', 'surat surat': 'surat', 'shreveport': 'shreveport', 'central michigan': 'centralmich', 'tucson': 'tucson', 'flint': 'flint', 'tuscarawas co': 'tuscarawas', 'ottawa-hull-gatineau': 'ottawa', 'western IL': 'quincy', 'ft myers / SW florida': 'fortmyers', 'wausau': 'wausau', 'imperial county': 'imperial', 'twin tiers NY/PA': 'twintiers', 'kingston': 'kingston', 'marseille': 'marseilles', 'sandusky': 'sandusky', 'taiwan': 'taipei', 'fairbanks': 'fairbanks', 'killeen / temple / ft hood': 'killeen', 'brittany': 'rennes', 'croatia': 'zagreb', 'amarillo': 'amarillo', 'birmingham': 'bham', 'whitehorse': 'whitehorse', 'green bay': 'greenbay', 'texarkana': 'texarkana', 'romania': 'bucharest', 'boulder': 'boulder', 'baja california sur': 'bajasur', 'columbus': 'columbus', 'denver': 'denver', 'modesto': 'modesto', 'greece': 'athens', 'stockton': 'stockton', 'meridian': 'meridian', 'los angeles': 'losangeles', 'hangzhou': 'hangzhou', 'frederick': 'frederick', 'london ': 'londonon', 'florence / muscle shoals': 'shoals', 'johannesburg': 'johannesburg', 'ahmedabad': 'ahmedabad', 'eastern kentucky': 'eastky', 'elko': 'elko', 'dubuque': 'dubuque', 'space coast': 'spacecoast', 'klamath falls': 'klamath', 'yuma': 'yuma', 'kalamazoo': 'kalamazoo', 'southwest TX': 'bigbend', 'hampton roads': 'norfolk', 'jaipur': 'jaipur', 'lima / findlay': 'limaohio', 'mankato': 'mankato', 'recife': 'recife', 'gold coast': 'goldcoast', 'geneva': 'geneva', 'hampshire': 'hampshire', 'butte': 'butte', 'charleston': 'charleston', 'hamilton-burlington': 'hamilton', 'fort dodge': 'fortdodge', 'hong kong': 'hongkong', 'altoona-johnstown': 'altoona', 'genoa': 'genoa', 'hawaii': 'honolulu', 'beijing': 'beijing', 'northern WI': 'northernwi', 'budapest': 'budapest', 'reykjavik': 'reykjavik', 'lucknow': 'lucknow', 'malaga': 'malaga', 'harrisonburg': 'harrisonburg', 'canarias': 'canarias', 'savannah / hinesville': 'savannah', 'guelph': 'guelph', 'medicine hat': 'hat', 'kalispell': 'kalispell', 'youngstown': 'youngstown', 'monterrey': 'monterrey', 'mumbai': 'mumbai', 'southern WV': 'swv', 'stuttgart': 'stuttgart', 'dundee': 'dundee', 'ft mcmurray': 'ftmcmurray', 'kolkata (calcutta)': 'kolkata', 'cologne': 'cologne', 'guangzhou': 'guangzhou', 'basel': 'basel', 'durban': 'durban', 'sevilla': 'sevilla', 'perugia': 'perugia', 'charlottesville': 'charlottesville', 'davao city': 'davaocity', 'western maryland': 'westmd', 'zamboanga': 'zamboanga', 'pierre / central SD': 'csd', "st john's": 'newfoundland', 'eastern panhandle': 'martinsburg', 'louisville': 'louisville', 'charleston ': 'charlestonwv', 'appleton-oshkosh-FDL': 'appleton', 'sheboygan': 'sheboygan', 'topeka': 'topeka', 'hermosillo': 'hermosillo', 'northwest KS': 'nwks', 'wilmington': 'wilmington', 'halifax': 'halifax', 'battle creek': 'battlecreek', 'shanghai': 'shanghai', 'rochester ': 'rmn', 'ethiopia': 'addisababa', 'billings': 'billings', 'tallahassee': 'tallahassee', 'bremen': 'bremen', 'monroe': 'monroe', 'yucatan': 'yucatan', 'nicaragua': 'managua', 'sherbrooke': 'sherbrooke', 'jackson ': 'jxn', 'devon & cornwall': 'devon', 'ecuador': 'quito', 'peoria': 'peoria', 'miami / dade': 'miami', 'cadiz': 'cadiz', 'boise': 'boise', 'ithaca': 'ithaca', 'milan': 'milan', 'pretoria': 'pretoria', 'iran': 'tehran', 'skagit / island / SJI': 'skagit', 'guatemala': 'guatemala', 'akron / canton': 'akroncanton', 'scranton / wilkes-barre': 'scranton', 'mendocino county': 'mendocino', 'kenya': 'kenya', 'porto alegre': 'portoalegre', 'inland empire': 'inlandempire', 'dayton / springfield': 'dayton', 'puerto rico': 'puertorico', 'delaware': 'delaware', 'santa maria': 'santamaria', 'parkersburg-marietta': 'parkersburg', 'athens': 'athensga', 'sudbury': 'sudbury', 'frankfurt': 'frankfurt', 'valencia': 'valencia', 'calgary': 'calgary', 'bulgaria': 'bulgaria', 'palm beach county': 'miami', 'new brunswick': 'newbrunswick', 'mohave county': 'mohave', 'columbus ': 'columbusga', 'essen / ruhr': 'essen', 'wichita falls': 'wichitafalls', 'buffalo': 'buffalo', 'upper peninsula': 'up', 'montreal': 'montreal', 'regina': 'regina', 'munich': 'munich', 'shenzhen': 'shenzhen', 'vermont': 'vermont', 'rome': 'rome', 'milwaukee': 'milwaukee', 'northwest CT': 'nwct', 'belo horizonte': 'belohorizonte', 'canberra': 'canberra', 'prince edward island': 'pei', 'mobile': 'mobile', 'new haven': 'newhaven', 'moscow': 'moscow', 'cincinnati': 'cincinnati', 'sheffield': 'sheffield', 'norway': 'oslo', 'clovis / portales': 'clovis', 'saguenay': 'saguenay', 'colombia': 'colombia', 'wichita': 'wichita', 'birmingham / west mids': 'birmingham', 'dalian': 'dalian', 'indonesia': 'jakarta', 'shenyang': 'shenyang', 'pampanga': 'pampanga', 'mazatlan': 'mazatlan', 'kansas city': 'kansascity', 'new orleans': 'neworleans', 'paris': 'paris', 'poland': 'warsaw', 'port huron': 'porthuron', 'jonesboro': 'jonesboro', 'merced': 'merced', 'chautauqua': 'chautauqua', 'san antonio': 'sanantonio', 'fayetteville': 'fayetteville', 'kitchener-waterloo-cambridge': 'kitchener', 'college station': 'collegestation', 'winnipeg': 'winnipeg', 'glasgow': 'glasgow', 'melbourne': 'melbourne', 'moses lake': 'moseslake', 'terre haute': 'terrehaute', 'chillicothe': 'chillicothe', 'madison': 'madison', 'houston': 'houston', 'heidelberg': 'heidelberg', 'goa': 'goa', 'finger lakes': 'fingerlakes', 'memphis': 'memphis', 'joplin': 'joplin', 'berlin': 'berlin', 'fort wayne': 'fortwayne', 'kent': 'kent', 'cumberland valley': 'chambersburg', 'eastern CO': 'eastco', 'vienna': 'vienna', 'seattle-tacoma': 'seattle', 'southern maryland': 'smd', 'delhi': 'delhi', 'columbia / jeff city': 'columbiamo', 'puerto vallarta': 'pv', 'olympic peninsula': 'olympic', 'christchurch': 'christchurch', 'edmonton': 'edmonton', 'cookeville': 'cookeville', 'wenatchee': 'wenatchee', 'tampa bay area': 'tampa', 'waterloo / cedar falls': 'waterloo', 'dominican republic': 'santodomingo', 'montevideo': 'montevideo', 'puebla': 'puebla', 'north platte': 'northplatte', 'champaign urbana': 'chambana', 'waco': 'waco', 'okaloosa / walton': 'okaloosa', 'salt lake city': 'saltlakecity', 'eastern CT': 'newlondon', 'gainesville': 'gainesville', 'monterey bay': 'monterey', 'rochester': 'rochester', 'vancouver': 'vancouver', 'pune': 'pune', 'newcastle, NSW': 'ntl', 'southwest KS': 'swks', 'roseburg': 'roseburg', 'st augustine': 'staugustine', 'south florida': 'miami', 'bristol': 'bristol', 'kennewick-pasco-richland': 'kpr', 'syracuse': 'syracuse', 'cardiff / wales': 'cardiff', 'san diego': 'sandiego', 'holland': 'holland', 'quad cities': 'quadcities', 'san marcos': 'sanmarcos', 'finland': 'helsinki', 'binghamton': 'binghamton', 'lawton': 'lawton', 'oneonta': 'oneonta', 'bilbao': 'bilbao', 'dusseldorf': 'dusseldorf', 'pullman / moscow': 'pullman', 'belfast': 'belfast', 'grand forks': 'grandforks', 'mexico city': 'mexicocity', 'saskatoon': 'saskatoon', "nice / cote d'azur": 'cotedazur', 'pittsburgh': 'pittsburgh', 'cleveland': 'cleveland', 'myrtle beach': 'myrtlebeach', 'florida keys': 'keys', 'western massachusetts': 'westernmass', 'fayetteville ': 'fayar', 'hickory / lenoir': 'hickory', 'clarksville': 'clarksville', 'vietnam': 'vietnam', 'nanjing': 'nanjing', 'north jersey': 'newjersey', 'siskiyou county': 'siskiyou', 'worcester / central MA': 'worcester', 'sault ste marie': 'soo', 'tel aviv': 'telaviv', 'coventry': 'coventry', 'eugene': 'eugene', 'victoria': 'victoria', 'kenai peninsula': 'kenai', 'bowling green': 'bgky', 'lyon': 'lyon', 'indianapolis': 'indianapolis', 'monroe ': 'monroemi', 'hannover': 'hannover', 'winchester': 'winchester', 'derby': 'derby', 'adelaide': 'adelaide', 'chattanooga': 'chattanooga', 'hudson valley': 'hudsonvalley', 'lincoln': 'lincoln', 'owensboro': 'owensboro', 'guanajuato': 'guanajuato', 'bangalore': 'bangalore', 'kuwait': 'kuwait', 'haifa': 'haifa', 'des moines': 'desmoines', 'beirut, lebanon': 'beirut', 'odessa / midland': 'odessa', 'orlando': 'orlando', 'skeena-bulkley': 'skeena', 'prince george': 'princegeorge', 'wuhan': 'wuhan', 'dresden': 'dresden', 'tulsa': 'tulsa', 'veracruz': 'veracruz', 'united arab emirates': 'dubai', 'granada': 'granada', 'perth': 'perth', 'greenville / upstate': 'greenville', 'sioux city': 'siouxcity', 'bordeaux': 'bordeaux', 'windsor': 'windsor', 'jacksonville': 'jacksonville', 'costa rica': 'costarica', 'lisbon': 'lisbon', 'hamburg': 'hamburg', 'fort lauderdale': 'fortlauderdale', 'cairns': 'cairns', 'faro / algarve': 'faro', 'st george': 'stgeorge', 'evansville': 'evansville', 'tijuana': 'tijuana', 'high rockies': 'rockies', 'mcallen / edinburg': 'mcallen', 'laredo': 'laredo', 'owen sound': 'owensound', 'leipzig': 'leipzig', 'las cruces': 'lascruces', 'cape town': 'capetown', 'wellington': 'wellington', 'nanaimo': 'nanaimo', 'colorado springs': 'cosprings', 'buenos aires': 'buenosaires', 'fraser valley': 'abbotsford', 'brownsville': 'brownsville', 'fort smith': 'fortsmith', 'sierra vista': 'sierravista', 'osaka-kobe-kyoto': 'osaka', 'amsterdam / randstad': 'amsterdam', 'grand island': 'grandisland', 'bismarck': 'bismarck', 'kenosha-racine': 'racine', 'pensacola': 'pensacola', 'cape cod / islands': 'capecod', 'territories': 'territories', 'santa fe / taos': 'santafe', 'wyoming': 'wyoming', 'texoma': 'texoma', 'atlanta': 'atlanta', 'pueblo': 'pueblo', 'anchorage / mat-su': 'anchorage', 'heartland florida': 'cfl', 'glens falls': 'glensfalls', 'madrid': 'madrid', 'gadsden-anniston': 'gadsden', 'jersey shore': 'jerseyshore', 'chandigarh': 'chandigarh', 'lille': 'lille', 'yellowknife': 'yellowknife', 'east anglia': 'norwich', 'mansfield': 'mansfield', 'hiroshima': 'hiroshima', 'lafayette / west lafayette': 'tippecanoe', 'belleville': 'belleville', 'peace river country': 'peace', 'grand rapids': 'grandrapids', 'north mississippi': 'northmiss', 'harrisburg': 'harrisburg', 'sweden': 'stockholm', 'eastern montana': 'montana', 'salvador, bahia': 'salvador', 'okinawa': 'okinawa', 'wollongong': 'wollongong', 'st petersburg': 'stpetersburg', 'athens ': 'athensohio', 'barrie': 'barrie', 'reading': 'reading', 'missoula': 'missoula', 'potsdam-canton-massena': 'potsdam', 'brighton': 'brighton', 'bolivia': 'lapaz', 'great falls': 'greatfalls', 'morgantown': 'morgantown', 'bloomington-normal': 'bn', 'chennai (madras)': 'chennai', 'macon / warner robins': 'macon', 'lawrence': 'lawrence', 'ashtabula': 'ashtabula', 'bern': 'bern', 'broward county': 'miami', 'statesboro': 'statesboro', 'el salvador': 'elsalvador', 'nuremberg': 'nuremberg', 'nashville': 'nashville', 'brantford-woodstock': 'brantford', 'oregon coast': 'oregoncoast', 'eastern NC': 'eastnc', 'iloilo': 'iloilo', 'comox valley': 'comoxvalley', 'springfield': 'springfield', 'dunedin': 'dunedin', 'prescott': 'prescott', 'manchester': 'manchester', 'bangladesh': 'bangladesh', 'greensboro': 'greensboro', 'venice / veneto': 'venice', 'bicol region': 'naga', 'corvallis/albany': 'corvallis', 'south dakota': 'sd', 'redding': 'redding', 'leeds': 'leeds', 'orange county': 'orangecounty', 'alicante': 'alicante', 'sioux falls / SE SD': 'siouxfalls', 'janesville': 'janesville', 'annapolis': 'annapolis', 'victoria ': 'victoriatx', 'southeast alaska': 'juneau', 'ogden-clearfield': 'ogden', 'grenoble': 'grenoble', 'ann arbor': 'annarbor', 'quebec city': 'quebec', 'state college': 'pennstate', 'columbia': 'columbia', 'utica-rome-oneida': 'utica', 'brisbane': 'brisbane', 'northeast SD': 'nesd', 'thunder bay': 'thunderbay', 'chico': 'chico', 'danville': 'danville', 'montgomery': 'montgomery', 'florence': 'florencesc', 'peterborough': 'peterborough', 'bacolod': 'bacolod', 'hartford': 'hartford', 'erie': 'erie', 'baton rouge': 'batonrouge', 'york': 'york', 'ames': 'ames', 'decatur': 'decatur', 'long island': 'longisland', 'bologna': 'bologna', 'watertown': 'watertown', 'flagstaff / sedona': 'flagstaff', 'chihuahua': 'chihuahua', 'brunswick': 'brunswick', 'montpellier': 'montpellier', 'ukraine': 'ukraine', 'kerala': 'kerala', 'boston': 'boston', 'st cloud': 'stcloud', 'sacramento': 'sacramento', 'egypt': 'cairo', 'london': 'london', 'east midlands': 'eastmids', 'charlotte': 'charlotte', 'iowa city': 'iowacity', 'jacksonville ': 'onslow', 'iraq': 'baghdad', 'new river valley': 'blacksburg', 'whistler': 'whistler', 'kirksville': 'kirksville', 'southwest MN': 'marshall', 'palm springs': 'palmsprings', 'del rio / eagle pass': 'delrio', 'deep east texas': 'nacogdoches', 'copenhagen': 'copenhagen', 'newcastle / NE england': 'newcastle', 'toronto': 'toronto', 'detroit metro': 'detroit', 'western KY': 'westky', 'el paso': 'elpaso', 'manhattan': 'ksu', 'peru': 'lima', 'central NJ': 'cnj', 'southeast KS': 'seks', 'pakistan': 'pakistan', 'knoxville': 'knoxville', 'san francisco bay area': 'sfbay', 'logan': 'logan', 'morocco': 'casablanca', 'abilene': 'abilene', 'northern michigan': 'nmi', 'lynchburg': 'lynchburg', 'kelowna / okanagan': 'kelowna', 'tuscaloosa': 'tuscaloosa', 'normandy': 'rouen', 'jackson': 'jackson', 'north dakota': 'nd', 'yakima': 'yakima', 'cornwall': 'cornwall', 'southern illinois': 'carbondale', 'bozeman': 'bozeman', 'lake of the ozarks': 'loz', "spokane / coeur d'alene": 'spokane', 'rio de janeiro': 'rio', 'ocala': 'ocala', 'little rock': 'littlerock', 'tokyo': 'tokyo', 'edinburgh': 'edinburgh', 'indore': 'indore', 'bend': 'bend', 'guadalajara': 'guadalajara', 'lethbridge': 'lethbridge', 'southeast missouri': 'semo', 'boone': 'boone', 'malaysia': 'malaysia', 'bath': 'bath', 'maine': 'maine', 'bakersfield': 'bakersfield', 'hyderabad': 'hyderabad', 'portland': 'portland', 'rhode island': 'providence', 'auckland': 'auckland', 'minneapolis / st paul': 'minneapolis', 'barcelona': 'barcelona', 'southeast IA': 'ottumwa', 'roanoke': 'roanoke', 'southwest MS': 'natchez', 'williamsport': 'williamsport', 'roswell / carlsbad': 'roswell', 'las vegas': 'lasvegas', 'thailand': 'bangkok', 'san luis obispo': 'slo', 'la crosse': 'lacrosse', 'omaha / council bluffs': 'omaha', 'seoul': 'seoul', 'hanford-corcoran': 'hanford', 'sarnia': 'sarnia', 'elmira-corning': 'elmira', 'auburn': 'auburn', 'cedar rapids': 'cedarrapids', 'jackson  ': 'jacksontn', 'the thumb': 'thumb', 'la salle co': 'lasalle', 'florence / tuscany': 'florence', 'cebu': 'cebu', 'san angelo': 'sanangelo', 'torino': 'torino', 'santa barbara': 'santabarbara', 'turkey': 'istanbul', 'st louis': 'stlouis', 'chile': 'santiago', 'catskills': 'catskills', 'nottingham': 'nottingham', 'oxford': 'oxford'}

def scrape_cities():
    """Scrape city and link for dict of valid names."""
    r = requests.get("http://www.craigslist.org/about/sites")
    soup = BeautifulSoup(r.text, "html.parser")
    cities = []
    for node in soup.find_all('a'):
        if node.find_parent('li'):
            if (node['href'].startswith('//') and not node['href'].startswith('//www.')):
                city = ()
                link = node['href'][2:]
                link = link.split('.', 1)[0]
                city = node.get_text()
                cities.append((city, link))
    city_dict = {c[0]: c[1] for c in cities}
    return city_dict

        
if __name__ == '__main__':
    cities = scrape_cities()
    print(cities)
    
